package com.qmth.teachcloud.common.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.gson.Gson;
import com.qmth.teachcloud.common.bean.auth.AuthBean;
import com.qmth.teachcloud.common.bean.dto.MenuDto;
import com.qmth.teachcloud.common.bean.dto.MenuPrivilegeDto;
import com.qmth.teachcloud.common.bean.result.MenuResult;
import com.qmth.teachcloud.common.entity.SysRole;
import com.qmth.teachcloud.common.entity.SysRolePrivilege;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.entity.SysUserRole;
import com.qmth.teachcloud.common.enums.PrivilegeEnum;
import com.qmth.teachcloud.common.enums.RoleTypeEnum;
import com.qmth.teachcloud.common.mapper.SysUserRoleMapper;
import com.qmth.teachcloud.common.service.CacheService;
import com.qmth.teachcloud.common.service.SysRolePrivilegeService;
import com.qmth.teachcloud.common.service.SysUserRoleService;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * <p>
 * 用户和角色关联表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class SysUserRoleServiceImpl extends ServiceImpl<SysUserRoleMapper, SysUserRole> implements SysUserRoleService {

    @Resource
    CacheService cacheService;

    @Override
    public List<SysUserRole> listByRoleId(Long id) {
        QueryWrapper<SysUserRole> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysUserRole::getRoleId, id);
        return this.list(queryWrapper);
    }

    @Override
    public List<SysRole> listRoleByUserId(Long userId) {
        return this.baseMapper.listRoleByUserId(userId);
    }

    @Override
    public void removeByUserId(Long id) {
        UpdateWrapper<SysUserRole> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(SysUserRole::getUserId, id);
        this.remove(updateWrapper);
    }

    @Override
    public MenuResult listByUserId() {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        AuthBean authBean = cacheService.userAuthCache(sysUser.getId());
        List<MenuDto> menuDtoList = null;
        for (SysRole role : authBean.getRoleList()) {
            if (Objects.nonNull(role.getType()) && RoleTypeEnum.ADMIN.name().equals(role.getType().name())) {
                menuDtoList = this.baseMapper.listAll(Objects.isNull(sysUser.getSchoolId()) ? (Long) ServletUtil.getRequestSchoolByNotVaild() : sysUser.getSchoolId());
                break;
            }
        }
        if (Objects.isNull(menuDtoList)) {
            menuDtoList = this.baseMapper.listByUserId(sysUser.getId(), Objects.isNull(sysUser.getSchoolId()) ? (Long) ServletUtil.getRequestSchoolByNotVaild() : sysUser.getSchoolId());
        }
        LinkedMultiValueMap<Long, MenuDto> linkedMultiValueMap = new LinkedMultiValueMap<>();
        List<MenuPrivilegeDto> menuPrivilegeDtoList = new ArrayList<>();
        Gson gson = new Gson();
        for (MenuDto m : menuDtoList) {
            if (Objects.isNull(m.getParentId()) || m.getType() == PrivilegeEnum.MENU) {
                menuPrivilegeDtoList.add(gson.fromJson(gson.toJson(m), MenuPrivilegeDto.class));
            } else {
                if (!linkedMultiValueMap.containsKey(m.getParentId())) {
                    linkedMultiValueMap.add(m.getParentId(), m);
                } else {
                    List<MenuDto> menuDtos = linkedMultiValueMap.get(m.getParentId());
                    menuDtos.add(m);
                    linkedMultiValueMap.put(m.getParentId(), menuDtos);
                }
            }
        }
        for (MenuPrivilegeDto m : menuPrivilegeDtoList) {
            List<MenuDto> menuDtos = linkedMultiValueMap.get(m.getId());
            List<MenuDto> urls = null, buttons = null, links = null, lists = null, conditions = null;
            if (Objects.nonNull(menuDtos)) {
                for (MenuDto menuDto : menuDtos) {
                    if (menuDto.getType() == PrivilegeEnum.BUTTON) {
                        buttons = Objects.isNull(buttons) ? new ArrayList<>() : buttons;
                        buttons.add(gson.fromJson(gson.toJson(menuDto), MenuPrivilegeDto.class));
                        m.setButtons(buttons);
                    } else if (menuDto.getType() == PrivilegeEnum.LINK) {
                        links = Objects.isNull(links) ? new ArrayList<>() : links;
                        links.add(gson.fromJson(gson.toJson(menuDto), MenuPrivilegeDto.class));
                        m.setLinks(links);
                    } else if (menuDto.getType() == PrivilegeEnum.URL) {
                        urls = Objects.isNull(urls) ? new ArrayList<>() : urls;
                        urls.add(gson.fromJson(gson.toJson(menuDto), MenuPrivilegeDto.class));
                        m.setUrls(urls);
                    } else if (menuDto.getType() == PrivilegeEnum.LIST) {
                        lists = Objects.isNull(lists) ? new ArrayList<>() : lists;
                        lists.add(gson.fromJson(gson.toJson(menuDto), MenuPrivilegeDto.class));
                        m.setLists(lists);
                    } else if (menuDto.getType() == PrivilegeEnum.CONDITION) {
                        conditions = Objects.isNull(conditions) ? new ArrayList<>() : conditions;
                        conditions.add(gson.fromJson(gson.toJson(menuDto), MenuPrivilegeDto.class));
                        m.setConditions(conditions);
                    }
                }
            }
        }
        MenuResult menuResult = new MenuResult(sysUser.getId(), authBean.getRoleList().stream().map(s -> s.getName()).collect(Collectors.toSet()));
        menuResult.setPrivileges(menuPrivilegeDtoList);
        return menuResult;
    }

    @Override
    public void rebindPrivileges(Long roleId) {
        // 查询绑定该角色的所有用户
        List<SysUserRole> sysUserRoles = this.listByRoleId(roleId);
        Set<Long> userIds = sysUserRoles.stream().map(m -> m.getUserId()).collect(Collectors.toSet());
        //删除
        UpdateWrapper<SysUserRole> userRoleUpdateWrapper = new UpdateWrapper<>();
        userRoleUpdateWrapper.lambda().in(SysUserRole::getUserId, userIds);
        List<SysUserRole> list = new ArrayList<>();

        for (Long userId : userIds) {
            List<SysRole> sysRoles = this.listRoleByUserId(userId);
            for (SysRole role : sysRoles) {
                List<SysRolePrivilege> rolePrivileges = cacheService.rolePrivilegeCache(role.getId());
                for (SysRolePrivilege rolePrivilege : rolePrivileges) {
                    SysUserRole userRole = new SysUserRole();
                    userRole.setUserId(userId);
                    userRole.setRoleId(role.getId());
                    userRole.setPrivilegeId(rolePrivilege.getPrivilegeId());
                    list.add(userRole);
                }
            }

        }

        this.saveBatch(list);
    }
}
